//--------------------------------------------------------------------------
//
//  Software for MSP430 based e-meters.
//
//--------------------------------------------------------------------------
//
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <io.h>
#include "emeter-template.h"
#include "emeter-dlt645.h"
#include "emeter-app.h"

// Call RTC library
#include "emeter-rtc-inter.h"

extern void timeout_callback(void);
extern struct rtc_interface user_rtc;		
extern int temperature_C;

serial_msg_buf_t tx_msg[DLT645_PORT_SUPPORT_NUM];
serial_msg_buf_t rx_msg[DLT645_PORT_SUPPORT_NUM];
uint8_t receive_flag[DLT645_PORT_SUPPORT_NUM];
uint8_t ready_flag[DLT645_PORT_SUPPORT_NUM];
uint8_t overtime_flag[DLT645_PORT_SUPPORT_NUM];
uint16_t com_timeout[DLT645_PORT_SUPPORT_NUM];

const struct uart_register_addr_s uart_register_addr_const[] = 
{
#if DLT645_PORT_SUPPORT_1
	{
		&UCA0CTL0,
		&UCA0CTL1,
		&UCA0BR0,	
		&UCA0BR1,	
		&UCA0MCTL,
		&UCA0STAT,
		&UCA0RXBUF,
		&UCA0TXBUF,
		&UCA0IE,
		&UCA0IFG,
		&UCA0IV,
		//---------------------------
		&P1DIR,
		&P1OUT,
		&P1SEL,
		BIT3,		// txd
		BIT2,		// rxd
	},
#endif	
#if DLT645_PORT_SUPPORT_2
{
  &UCA1CTL0,
  &UCA1CTL1,
  &UCA1BR0,	
  &UCA1BR1,	
  &UCA1MCTLW,
  &UCA1STATW,
  &UCA1RXBUF,
  &UCA1TXBUF,
  &UCA1IE,
  &UCA1IFG,
  &UCA1IV,
  //---------------------------
  &P1DIR,
  &P1OUT,
  &P1SEL,
  BIT5,
  BIT4,
},
#endif
#if DLT645_PORT_SUPPORT_3
{
  &UCA2CTL0,
  &UCA2CTL1,
  &UCA2BR0, 
  &UCA2BR1, 
  &UCA2MCTLW,
  &UCA2STATW,
  &UCA2RXBUF,
  &UCA2TXBUF,
  &UCA2IE,
  &UCA2IFG,
  &UCA2IV,
  //---------------------------
  &P2DIR,
  &P2OUT,
  &P2SEL,
  BIT3,		//TXD
  BIT2,		//RXD
},
#endif	
#if DLT645_PORT_SUPPORT_4
{
  &UCA3CTL0,
  &UCA3CTL1,
  &UCA3BR0, 
  &UCA3BR1, 
  &UCA3MCTL,
  &UCA3STAT,
  &UCA3RXBUF,
  &UCA3TXBUF,
  &UCA3IE,
  &UCA3IFG,
  &UCA3IV,
  //---------------------------
  &P10DIR,
  &P10OUT,
  &P10SEL,
  BIT4,
  BIT5,
},
#endif	
};

// У
enum host_commands_e
{
  HOST_CMD_SET_RTC                            	= 0x01,
  HOST_CMD_GET_RTC															= 0x02,                            	
  HOST_CMD_GET_TEMPERATURE = 3,					
  HOST_CMD_SET_RTC_OFFSET                       = 0x04,			
};

static const uint8_t address[6] =
{
  0x99, 0x99, 0x99, 0x99, 0x99, 0x99
};

void send_message(int port, int len)
{
  tx_msg[port].ptr = 0;
  tx_msg[port].len = len;
	receive_flag[port] = 0;
  
	*uart_register_addr_const[port].pUCAxIFG |= UCTXIFG;
	*uart_register_addr_const[port].pUCAxIE &= ~UCRXIE; 
	*uart_register_addr_const[port].pUCAxIE |= UCTXIE; 
}

void recv_message(int port)
{
	*uart_register_addr_const[port].pUCAxIFG &= ~UCTXIFG;
	*uart_register_addr_const[port].pUCAxIFG &= ~UCRXIFG;	
	*uart_register_addr_const[port].pUCAxIE &= ~UCRXIE; 
	*uart_register_addr_const[port].pUCAxIE &= ~UCTXIE; 
	ready_flag[port] = 1;
	receive_flag[port] = 0;	
}


int prepare_tx_message(int port, int len)
{
  int i;
  uint8_t *s;
  
  s = (uint8_t *)tx_msg[port].buf.uint8;
  s[0] = 0xFE;
  s[1] = 0xFE;
  s[2] = 0xFE;
  s[3] = 0xFE;
  s[4] = 0x68;
  s[5] = address[0];
  s[6] = address[1];
  s[7] = address[2];
  s[8] = address[3];
  s[9] = address[4];
  s[10] = address[5];
  s[11] = 0x68;
  s[12] = 0x23;
  s[13] = len;
  s[DLT645_MSG_TX_START_BODY + len] = 0;
  s[DLT645_MSG_TX_START_BODY + len + 1] = 0x16;
  
  for (i = 4;  i < DLT645_MSG_TX_START_BODY + len;  i++)
    s[DLT645_MSG_TX_START_BODY + len] += s[i];
  len = DLT645_MSG_TX_START_BODY + len + 2;
  if (len > 4 + 12 + MAX_DLT645_MSG_BODY)
    return 0;
  send_message(port, len);
  return  1;
}

static void dlt645_process_rx_message(int port, serial_msg_t *rx_msg, int rx_len)
{
  struct rtc_interface tmp_user_rtc;
  serial_msg_buf_t *tx;
  uint8_t *tx8;
  uint16_t *tx16;
  
  tx = (serial_msg_buf_t*)&tx_msg[port];
  tx8 = tx->buf.uint8;
  tx16 = tx->buf.uint16;
  
  /* Messages with type 0x23 are custom messages we
  use for calibration, password protection, etc.
  All other message types go to a custom message
  handler (if available). */
  if (rx_msg->uint8[8] != 0x23)
  {
    return;
  }
  
  switch (rx_msg->uint8[DLT645_MSG_RX_START_BODY])
  {
    //----------------------------------------------------------------------------------      
  case HOST_CMD_SET_RTC:    
    tmp_user_rtc.second = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 7];
    tmp_user_rtc.minute = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 6];
    tmp_user_rtc.hour = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 5]; 		
    tmp_user_rtc.day = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 4];
    tmp_user_rtc.month = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 3];
    tmp_user_rtc.year = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 2];
    
    set_rtc_parameter(RTC_CURRENT_TIME, (int32_t)&tmp_user_rtc);      
    
    tx8[DLT645_MSG_TX_START_BODY] = rx_msg->uint8[DLT645_MSG_RX_START_BODY];
    tx8[DLT645_MSG_TX_START_BODY + 1] = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 1] | 0x80;
    prepare_tx_message(port, 2);
    break;
    //----------------------------------------------------------------------------------             
  case HOST_CMD_SET_RTC_OFFSET:
    // ʼƫ, УRTCʱʹ 
    set_rtc_parameter(RTC_CRYSTAL_BASE_OFFSET,rx_msg->uint16[DLT645_MSG_RX_START_BODY_W + 1]);  
    
    tx8[DLT645_MSG_TX_START_BODY] = rx_msg->uint8[DLT645_MSG_RX_START_BODY];
    tx8[DLT645_MSG_TX_START_BODY + 1] = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 1] | 0x80;
    prepare_tx_message(port, 2);        
    break;
    //----------------------------------------------------------------------------------            
  case HOST_CMD_GET_RTC:
    tx8[DLT645_MSG_TX_START_BODY + 2] = user_rtc.year;
    tx8[DLT645_MSG_TX_START_BODY + 3] = user_rtc.month;
    tx8[DLT645_MSG_TX_START_BODY + 4] = user_rtc.day;
    tx8[DLT645_MSG_TX_START_BODY + 5] = user_rtc.hour;
    tx8[DLT645_MSG_TX_START_BODY + 6] = user_rtc.minute;
    tx8[DLT645_MSG_TX_START_BODY + 7] = user_rtc.second;
    tx8[DLT645_MSG_TX_START_BODY + 8] = user_rtc.week;
    
    tx8[DLT645_MSG_TX_START_BODY] = rx_msg->uint8[DLT645_MSG_RX_START_BODY];
    tx8[DLT645_MSG_TX_START_BODY + 1] = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 1] | 0x80;
    prepare_tx_message(port, 9);
    break;
    //----------------------------------------------------------------------------------        
  case HOST_CMD_GET_TEMPERATURE:
    tx16[DLT645_MSG_TX_START_BODY_W + 1] = temperature_C;
    
    tx8[DLT645_MSG_TX_START_BODY] = rx_msg->uint8[DLT645_MSG_RX_START_BODY];
    tx8[DLT645_MSG_TX_START_BODY + 1] = rx_msg->uint8[DLT645_MSG_RX_START_BODY + 1] | 0x80;
    prepare_tx_message(port, 4);
    break;
    //----------------------------------------------------------------------------------              		
  default:
    /* For all other message types reply with type 0xFF - bad message type */
    tx8[DLT645_MSG_TX_START_BODY] = rx_msg->uint8[DLT645_MSG_RX_START_BODY];
    tx8[DLT645_MSG_TX_START_BODY + 1] = 0xFF;
    prepare_tx_message(port, 2);
    break;
  }
}


//int debug1,debug2,debug3;
//uint8_t temp_b[40];
void dlt645_rx_byte(int port, uint8_t ch)
{
  int i;
  int sum;
  
  //rx_msg[port].inter_char_timeout = INTER_CHAR_TIMEOUT;
  com_timeout[port] = INTER_CHAR_TIMEOUT;
  if (rx_msg[port].ptr == 0)
  {
    if (ch == 0x68)
    {
      rx_msg[port].buf.uint8[rx_msg[port].ptr++] = ch;
      rx_msg[port].len = 12 + MAX_DLT645_MSG_BODY;
    }
  }
  else
  {
    if (rx_msg[port].ptr == 9)
    {
      if (ch <= MAX_DLT645_MSG_BODY)
        rx_msg[port].len = 12 + ch;
      else
        rx_msg[port].ptr = 0;
    }
    rx_msg[port].buf.uint8[rx_msg[port].ptr++] = ch;
    if (rx_msg[port].ptr == rx_msg[port].len)
    {
      /* End of message */
      com_timeout[port] = 0;
      
      //++debug1;
      sum = rx_msg[port].buf.uint8[0];
      for (i = 1;  i < rx_msg[port].len - 2;  i++)
        sum += rx_msg[port].buf.uint8[i];
      if (//rx_msg[port].buf.uint8[rx_msg[port].len - 2] == (sum & 0xFF)
          //&&
          rx_msg[port].buf.uint8[rx_msg[port].len - 1] == 0x16)
      {
        /* Good message received */
        //dlt645_process_rx_message(port, (serial_msg_t *)&rx_msg[port].buf, rx_msg[port].len);
        
        recv_message(port);
        com_timeout[port] = SEND_TIMEOUT;
        
        //++debug2;
      } 
#if 0
      else {
        //++debug3;
        for (i = 0;  i < rx_msg[port].len;  i++)
          temp_b[i] = rx_msg[port].buf.uint8[i];
      } 
#endif
      rx_msg[port].ptr = 0;
    }
  }
}



#if DLT645_PORT_SUPPORT_1
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	uint8_t data;
	uint8_t UCAxIFG_tmp = UCA0IFG;
	
	switch(__even_in_range(UCA0IV,4))
	{
  case 0:break; 														// Vector 0 - no interrupt
  case 2: 																	// Vector 2 - RXIFG 	high level
    data = UCA0RXBUF;
    if (receive_flag[DLT645_PORT1])
    { 
      dlt645_rx_byte(DLT645_PORT1,data);
      break;
    }
  case 4:
    if (UCAxIFG_tmp & UCTXIFG)
    {
      UCA0IFG &= ~UCTXIFG;
      
      UCA0TXBUF = tx_msg[DLT645_PORT1].buf.uint8[tx_msg[DLT645_PORT1].ptr++];
      if (tx_msg[DLT645_PORT1].ptr >= tx_msg[DLT645_PORT1].len)
      {
        /* Stop transmission */
        UCA0IE &= ~UCTXIE;		
        UCA0IE |= UCRXIE; 
        
        tx_msg[DLT645_PORT1].ptr = 0;
        tx_msg[DLT645_PORT1].len = 0;
        receive_flag[DLT645_PORT1] = 1;
      }
    }
    break;									 
  default: break;
	}  
}

#endif

#if DLT645_PORT_SUPPORT_2
#pragma vector=USCI_A1_VECTOR
__interrupt void USCI_A1_ISR(void)
{
	uint8_t data;
	uint8_t UCAxIFG_tmp = UCA1IFG;
	
	switch(__even_in_range(UCA1IV,4))
	{
  case 0:break; 														// Vector 0 - no interrupt
  case 2: 																	// Vector 2 - RXIFG 	high level
    data = UCA1RXBUF;
    if (receive_flag[DLT645_PORT2])
    { 
      dlt645_rx_byte(DLT645_PORT2,data);
      break;
    }
  case 4:
    if (UCAxIFG_tmp & UCTXIFG)
    {
      UCA1IFG &= ~UCTXIFG;
      
      UCA1TXBUF = tx_msg[DLT645_PORT2].buf.uint8[tx_msg[DLT645_PORT2].ptr++];
      if (tx_msg[DLT645_PORT2].ptr >= tx_msg[DLT645_PORT2].len)
      {
        /* Stop transmission */
        UCA1IE &= ~UCTXIE;		
        UCA1IE |= UCRXIE; 
        
        tx_msg[DLT645_PORT2].ptr = 0;
        tx_msg[DLT645_PORT2].len = 0;
        receive_flag[DLT645_PORT2] = 1;
      }
    }
    break;									 
  default: break;
	}  
}

#endif

#if DLT645_PORT_SUPPORT_3
#pragma vector=USCI_A2_VECTOR
__interrupt void USCI_A2_ISR(void)
{
	uint8_t data;
	uint8_t UCAxIFG_tmp = UCA2IFG;
	
	switch(__even_in_range(UCA2IV,4))
	{
  case 0:break; 														// Vector 0 - no interrupt
  case 2: 																	// Vector 2 - RXIFG 	high level
    data = UCA2RXBUF;
    //-------------------------------------      
    //UCA2TXBUF = data;
    //return;
    //-------------------------------------      
    if (receive_flag[DLT645_PORT3])
    { 
      dlt645_rx_byte(DLT645_PORT3,data);
      break;
    }
  case 4:
    if (UCAxIFG_tmp & UCTXIFG)
    {
      UCA2IFG &= ~UCTXIFG;
      
      UCA2TXBUF = tx_msg[DLT645_PORT3].buf.uint8[tx_msg[DLT645_PORT3].ptr++];
      if (tx_msg[DLT645_PORT3].ptr >= tx_msg[DLT645_PORT3].len)
      {
        /* Stop transmission */
        UCA2IE &= ~UCTXIE;		
        UCA2IE |= UCRXIE; 
        
        tx_msg[DLT645_PORT3].ptr = 0;
        tx_msg[DLT645_PORT3].len = 0;
        receive_flag[DLT645_PORT3] = 1;
      }
    }
    break;									 
  default: break;
	}  
}

#endif

#if DLT645_PORT_SUPPORT_4
#pragma vector=USCI_A3_VECTOR
__interrupt void USCI_A3_ISR(void)
{
	uint8_t data;
	uint8_t UCAxIFG_tmp = UCA3IFG;
  
	switch(__even_in_range(UCA3IV,4))
	{
  case 0:break;                             // Vector 0 - no interrupt
  case 2:                                   // Vector 2 - RXIFG		high level
    data = UCA3RXBUF;
    if (receive_flag[DLT645_PORT4])
    {	
      dlt645_rx_byte(DLT645_PORT4,data);
      break;
    }
  case 4:
    if (UCAxIFG_tmp & UCTXIFG)
    {
      UCA3IFG &= ~UCTXIFG;
      
      UCA3TXBUF = tx_msg[DLT645_PORT4].buf.uint8[tx_msg[DLT645_PORT4].ptr++];
      if (tx_msg[DLT645_PORT4].ptr >= tx_msg[DLT645_PORT4].len)
      {
        /* Stop transmission */
        UCA3IE &= ~UCTXIE;		
        UCA3IE |= UCRXIE; 
        
        tx_msg[DLT645_PORT4].ptr = 0;
        tx_msg[DLT645_PORT4].len = 0;
        receive_flag[DLT645_PORT4] = 1;
      }
    }
    break;                   
  default: break;
	}  
}
#endif

void  uart_hardware_init(const struct uart_register_addr_s *uart_register_addr)
{	
	// initilize the io port	
	*(uart_register_addr->pPxDIR) |=	uart_register_addr->PxTX;
	*(uart_register_addr->pPxDIR) &= ~uart_register_addr->PxRX;
	*(uart_register_addr->pPxOUT) |=	uart_register_addr->PxTX + uart_register_addr->PxRX;
	*(uart_register_addr->pPxSEL) |=	uart_register_addr->PxTX + uart_register_addr->PxRX;
  
	// uart initilize 
	*(uart_register_addr->pUCAxCTL1) |= UCSWRST;	 // holding...  
  
#if 0	
	*(uart_register_addr->pUCAxCTL1) |= UCSSEL_1; 										// 1 :CLK = ACLK 2:CLK = SMCLK
#if (UART_BAUD_RATE == 9600)	
	*(uart_register_addr->pUCAxBR0) = 0x03; 													// 32kHz/9600=3.41 (see User's Guide)
	*(uart_register_addr->pUCAxBR1) = 0x00; 												 
	*(uart_register_addr->pUCAxMCTL) = ((uint16_t) 0x92 << 8);			// Modulation 
#else //UART_BAUD_RATE = 2400
	*(uart_register_addr->pUCAxBR0) = 0x0D; 									// 32kHz/2400=3.41 (see User's Guide)
	*(uart_register_addr->pUCAxBR1) = 0x00; 												 
	*(uart_register_addr->pUCAxMCTL) = ((uint16_t) 0xB6 << 8);			// Modulation 
#endif	
#else
	*(uart_register_addr->pUCAxCTL1) |= UCSSEL1; 										/* UCLK = SMCLK */
#if (UART_BAUD_RATE == 9600)	
	//*(uart_register_addr->pUCAxBR0) = 0xD3; 													// 2048kHz/9600=3.41 (see User's Guide)
	//*(uart_register_addr->pUCAxBR1) = 0x06; 												 
	//*(uart_register_addr->pUCAxMCTL) = ((uint16_t) /*0xB5*/ 0xD6 << 8);			// Modulation 
  
	*(uart_register_addr->pUCAxBR0) = 0xDA; 													// 2048kHz/9600=3.41 (see User's Guide)
	*(uart_register_addr->pUCAxBR1) = 0x00; 												 
	*(uart_register_addr->pUCAxMCTL) = ((uint16_t) 0x55 << 8); 		// Modulation 
	
#else //UART_BAUD_RATE = 2400
	*(uart_register_addr->pUCAxBR0) = 0x4E; 									// 32kHz/2400=3.41 (see User's Guide)
	*(uart_register_addr->pUCAxBR1) = 0x1B; 												 
	*(uart_register_addr->pUCAxMCTL) = ((uint16_t) 0xAA << 8);			// Modulation 
#endif	
  
#endif
  //---------------------
#if 0
	UCA2CTL1 |= UCSSEL1;										 /* UCLK = SMCLK */
	UCA2BR1 = 0x06;
	UCA2BR0 = 0xD3;
  
	UCA2MCTLW = ((uint16_t) 0xD6 << 8);
#endif	
  //---------------------
	*(uart_register_addr->pUCAxCTL1) &= ~UCSWRST;  // releasing...
	//--------------------------	
	//uart_register_addr.pUCAxCTL0 |= UCMODE_1; 	// idle start
	//--------------------------
#if 1
	//*(uart_register_addr->pUCAxIFG) &= ~UCTXIFG;
	//*(uart_register_addr->pUCAxIFG) &= ~UCRXIFG;
	*(uart_register_addr->pUCAxIE) &= ~UCTXIE;	
	*(uart_register_addr->pUCAxIE) |= UCRXIE;	
#endif
  
#if 0
	*(uart_register_addr->pUCAxIFG) &= ~UCTXIFG;
	*(uart_register_addr->pUCAxIFG) &= ~UCRXIFG;
	*(uart_register_addr->pUCAxIE) &= ~UCTXIE;	
	*(uart_register_addr->pUCAxIE) &= UCRXIE;	
  
	*(uart_register_addr->pUCAxIE) |= UCTXIE;
	*(uart_register_addr->pUCAxIFG) |= UCTXIFG;
	__enable_interrupt();
#endif
}

void dlt645_init(void)
{
	kick_watchdog();
  
	set_rtc_parameter(RTC_1S_INTR_CB,(int32_t)timeout_callback); 	
#if DLT645_PORT_SUPPORT_1
	memset((serial_msg_t *)&tx_msg[DLT645_PORT1], 0, sizeof(serial_msg_buf_t));
	memset((serial_msg_t *)&rx_msg[DLT645_PORT1], 0, sizeof(serial_msg_buf_t));	
  
	uart_hardware_init(&uart_register_addr_const[DLT645_PORT1]);
	receive_flag[DLT645_PORT1] = 1;
#endif
#if DLT645_PORT_SUPPORT_2
	memset((serial_msg_t *)&tx_msg[DLT645_PORT2], 0, sizeof(serial_msg_buf_t));
	memset((serial_msg_t *)&rx_msg[DLT645_PORT2], 0, sizeof(serial_msg_buf_t));	
  
	uart_hardware_init(&uart_register_addr_const[DLT645_PORT2]);
	receive_flag[DLT645_PORT2] = 1;	
#endif
#if DLT645_PORT_SUPPORT_3
	memset(&tx_msg[DLT645_PORT3], 0, sizeof(serial_msg_buf_t));
	memset(&rx_msg[DLT645_PORT3], 0, sizeof(serial_msg_buf_t));	
  
#if 1		//Not Clou
  PMAPPWD = 0x02D52;                        // Enable Write-access to modify port mapping registers
	P2MAP2 = PM_UCA2TXD;
	P2MAP3 = PM_UCA2RXD;
  PMAPPWD = 0;                              // Disable Write-Access to modify port mapping registers
#endif
  
	uart_hardware_init(&uart_register_addr_const[DLT645_PORT3]);
	receive_flag[DLT645_PORT3] = 1;	
#endif
#if DLT645_PORT_SUPPORT_4
	memset(&tx_msg[DLT645_PORT4], 0, sizeof(serial_msg_buf_t));
	memset(&rx_msg[DLT645_PORT4], 0, sizeof(serial_msg_buf_t));	
  
	uart_hardware_init(&uart_register_addr_const[DLT645_PORT4]);
	receive_flag[DLT645_PORT4] = 1;		
#endif
  
#if 0
	while (1)
	{
    kick_watchdog();
		while (!(UCA2IFG & UCTXIFG));
    UCA2TXBUF = 0x00;	
	}
#endif
}


void timeout_callback(void)
{
	int loop;
	
	for (loop = 0; loop < DLT645_PORT_SUPPORT_NUM; loop++)
	{
		if (com_timeout[loop]) 
		{		
			if (!(--com_timeout[loop]))
				overtime_flag[loop] = 1;
		} 
	}
}

void protocol_time_handle(void)
{
	int loop;
  istate_t state; 
  
	for (loop = 0; loop < DLT645_PORT_SUPPORT_NUM; loop++)
	{
		if (overtime_flag[loop])
		{
      state = __get_interrupt_state();
      __disable_interrupt(); 		
			memset((serial_msg_t *)&tx_msg[loop], 0, sizeof(serial_msg_buf_t));
			memset((serial_msg_t *)&rx_msg[loop], 0, sizeof(serial_msg_buf_t)); 			
			uart_hardware_init(&uart_register_addr_const[loop]);
			receive_flag[loop] = 1;
			overtime_flag[loop] = 0;		
      __set_interrupt_state(state);					
		}
	}	
}


void dlt645_callback(void)
{
	int loop;
	
	for (loop = 0; loop < DLT645_PORT_SUPPORT_NUM; loop++)
	{
		if (ready_flag[loop])
		{
			dlt645_process_rx_message(loop, (serial_msg_t *)&rx_msg[loop].buf, rx_msg[loop].len);
			ready_flag[loop] = 0;		
		}
	}
  
	protocol_time_handle();
  
}

