//******************************************************************************
//  MSP430F6736 RTC Calibration Library for China Smart Grid
// 	
//  Texas Instruments Inc.
//******************************* ***********************************************

#include "msp430f6736.h"
#include <stdint.h>
#include "emeter-rtc-lib.h"


//--------------------------------------------------------------------------------------------------------------------
// 1s smclk cycles
#define CRYSTAL_BASE_CYCLES_PER_SECOND          1048576
// #define CRYSTAL_BASE_CYCLES_PER_SECOND          1048576 * 4  // Todo : for more accuracy

/* These values would be calibrated, and stored in info memory in a real application */
/* This value is the reading for the ADC when measuring the temperature diode at zero
degrees Celcius. It varies somewhat from sample to sample of the MSP430, but is
easy to calibrate at the equipment factory. It is merely necessary to measure the
diode at room temperature, and allow for the slope of the diode's characteric to
estimate the value at zero degrees Celcius. */
#define SMCLK_PER_32K_ACLK                      32

//this is the roughly number of aclk per second, not in ppm accuracy
#define ACLK_PER_SECOND 												(CRYSTAL_BASE_CYCLES_PER_SECOND/SMCLK_PER_32K_ACLK-1)
#define ACLK_PER_SECOND_RANGE										8  	// 240/SMCLK_PER_32K_ACLK
#define TIME_FOR_BAD_DELAY											64  // ACLK
//--------------------------------------------------------------------------------------------------------------------
// Current temperature and adc value
int16_t raw_temperature_from_adc;           
int16_t temperature_in_celsius;

int16_t raw_battery_from_adc;
int16_t raw_dvcc_from_adc;

#define TEMPERATURE_START			0x01
#define BATTERY_START					0x02
#define DVCC_START						0x04

uint8_t adc_start_flag;
//--------------------------------------------------------------------------------------------------------------------
/* This is the current estimate for the parts per million of error in 32*32768Hz clock. 
positive value means make pulse slower, negtive value means fasten pulse. */
static int16_t current_rtc_correction;

/* This is the current accumulated error in PPM */
static int16_t current_accumulate_error;
static int16_t old_accumulate_error;

#if defined(SOFTWARE_RTC_SUPPORT)
static int32_t integrated_rtc_correction;
#endif

/* This is number of ACLK cycles estimated to occupy 1 second */
static uint16_t estimated_num_of_aclk_per_second;
static uint16_t half_num_of_aclk_per_second;

#define DEBUG_FALL_PULSE  0
#if DEBUG_FALL_PULSE
uint16_t half_num_of_aclk_per_second_a[40];
int16_t old_accumulate_error_a[40];
int32_t gap_value[40];
int16_t test_loop;

uint16_t num_of_aclk_per_second_a[40];
int16_t accumulate_error_a[40];
int16_t accumulate_error_a1[40];
int16_t test_loop1;
#endif

#if (LIB_INFO_COMBINE_EN == 0)
__infomem__a_ extern const int16_t info_rtc[64] = {
														CRYSTAL_PPM_OFFSET,
														
														CRYSTAL_QUADRATIC_COEFF0,
														CRYSTAL_QUADRATIC_COEFF1,
														CRYSTAL_QUADRATIC_COEFF2,	//
														CRYSTAL_QUADRATIC_COEFF3,
														CRYSTAL_QUADRATIC_COEFF4,
														CRYSTAL_QUADRATIC_COEFF5,
};
#endif

#define	TEMPERATURE_1_AREA	-36		//-18
#define	TEMPERATURE_2_AREA	0			//0
#define	TEMPERATURE_3_AREA	96		//48
#define	TEMPERATURE_4_AREA	116		//58
#define	TEMPERATURE_5_AREA	136		//68


//--------------------------------------------------------------------------------------------------------------------
// status about rtc, init in the low_level_init
__no_init uint8_t lib_rtc_status;
//--------------------------------------------------------------------------------------------------------------------
extern void null_low_level_init(void);

//user_callback other_low_level_init = null_low_level_init;
user_callback rtc_intr_callback = null_low_level_init;
user_callback open_pin_supply = null_low_level_init;
user_callback close_pin_supply = null_low_level_init;
//user_callback pulse_pin_setting = null_low_level_init;

void null_low_level_init(void)
{
	return;
}
//------------------------------------------------------------------------------------------------
void adjust_halfs_pulse(void)
{
	int32_t half_accumulate_error;

  half_accumulate_error = current_accumulate_error + ((CRYSTAL_BASE_CYCLES_PER_SECOND + current_rtc_correction)>>1);
	old_accumulate_error = half_accumulate_error & 0x1F;
	half_num_of_aclk_per_second = (half_accumulate_error >> 5)-1;

	// half aclk	 
	TA0CCR2 = half_num_of_aclk_per_second - TIME_FOR_BAD_DELAY;
}

void adjust_1s_pulse(void)
{		
  current_accumulate_error += current_rtc_correction;  
  estimated_num_of_aclk_per_second = ACLK_PER_SECOND;
  
  while(current_accumulate_error >= SMCLK_PER_32K_ACLK) {
    estimated_num_of_aclk_per_second += 1;
    current_accumulate_error -= SMCLK_PER_32K_ACLK;
  }
  while(current_accumulate_error <= -SMCLK_PER_32K_ACLK) {
    estimated_num_of_aclk_per_second -= 1;
    current_accumulate_error += SMCLK_PER_32K_ACLK;
  }

  TA0CCR0 = estimated_num_of_aclk_per_second;            
  TA0CCR1 = estimated_num_of_aclk_per_second - TIME_FOR_BAD_DELAY;   
}
void smooth_temperature(int16_t temperature)
{ 
	if (adc_start_flag & TEMPERATURE_START)
  	raw_temperature_from_adc = raw_temperature_from_adc - (raw_temperature_from_adc >> 3) + temperature;
	else
	{
		adc_start_flag |= TEMPERATURE_START;
		raw_temperature_from_adc = (temperature << 3);
	}
}

void smooth_battery_adc(int16_t battery_adc)
{
	if (adc_start_flag & BATTERY_START)
  	raw_battery_from_adc = raw_battery_from_adc - (raw_battery_from_adc >> 3) + battery_adc;	
  else
  {
		adc_start_flag |= BATTERY_START;  
		raw_battery_from_adc = (battery_adc << 3);		
	}
}

void smooth_DVCC_adc(int16_t dvcc_adc)
{
	if (adc_start_flag & DVCC_START)
  	raw_dvcc_from_adc = raw_dvcc_from_adc - (raw_dvcc_from_adc >> 3) + dvcc_adc;
  else
  {
		adc_start_flag |= DVCC_START;  
		raw_dvcc_from_adc = (dvcc_adc << 3);				
	}
}
//--------------------------------------------------------------------------------------
void rtc_temperature_compensation_estimate(void)
{
  int32_t temperature;
  int16_t temperature1,temperature2; 
  
  temperature = temperature_measurement();
  
  // 0.5 unit
  temperature_in_celsius = temperature >> 3;
  
  /* Now we need to calculate the ppm of clock error due to the current temperature. */
  /* The crystal frequency varies in a parabolic manner with temperature. The exact
  curve for the crystal is defined by the manufacturers, so we use a centre point
  temperature and quadratic coefficient from the crystal's data sheet. */
  /* Subtract the centre point of the crystal curve. */
  temperature -= CRYSTAL_QUADRATIC_CENTRE_TEMPERATURE;
  
  /* Do the parabolic curve calculation, to find the current ppm of
  error due to temperature. */
  temperature = temperature*temperature;
  temperature >>= 4;//avoid overflow
  
#if CRYSTAL_QUADRATIC_COEFF_SETTING
	if (temperature_in_celsius <= TEMPERATURE_1_AREA)  // -20*2
		temperature = (temperature*CRYSTAL_QUADRATIC_COEFF[0]) >> 20; // PPM
	else 	if (temperature_in_celsius < TEMPERATURE_2_AREA)  // -0*2
		temperature = (temperature*CRYSTAL_QUADRATIC_COEFF[1]) >> 20; // PPM
//----------------------------------------------------------------------------------------		
	else 	if (temperature_in_celsius < TEMPERATURE_3_AREA)  // 50*2
		temperature = (temperature*CRYSTAL_QUADRATIC_COEFF[2]) >> 20; // PPM
//----------------------------------------------------------------------------------------			
	else 	if (temperature_in_celsius < TEMPERATURE_4_AREA)  // 60*2
		temperature = (temperature*CRYSTAL_QUADRATIC_COEFF[3]) >> 20; // PPM
	else if (temperature_in_celsius < TEMPERATURE_5_AREA)  // 70*2
		temperature = (temperature*CRYSTAL_QUADRATIC_COEFF[4]) >> 20; // PPM	
	else 
		temperature = (temperature*CRYSTAL_QUADRATIC_COEFF[5]) >> 20; // PPM			
#else
  temperature = (temperature*CRYSTAL_QUADRATIC_COEFF2) >> 20; // PPM
#endif  
	
	temperature1 = ((68719UL * temperature) >> 16);
  //Hardware RTC module calibration  
  temperature2 = temperature1;
  if(temperature2 > 240)
    temperature2 = 240;

  for(unsigned char i = 0; i < 10; i++)
  {
    if((RTCTCMP & RTCTCRDY))
    {
      RTCTCMP = 0x8000 + temperature2;        
      break;
    } 
  }
 
  /* In a real application we would just use option 3 in the following case statement.
  However, for demonstration purposes it is useful to be able to select between no
  compensation, compensating only for temperature, only for manufacturing error,
  or the combination of the two. */    
  current_rtc_correction = crystal_base_error - temperature1; 
  //current_rtc_correction =  - temperature1; 
  //current_rtc_correction = 0;
  //current_rtc_correction = crystal_base_error;
  if(current_rtc_correction > 240)
    current_rtc_correction = 240;
  if(current_rtc_correction < -240)
    current_rtc_correction = -240;
  
}


//--------------------------------------------------------------------------------------------------------------------
void rtc_lib_enter_lpm(void)
{
	istate_t state; 
	state = __get_interrupt_state();
	__disable_interrupt();	

	lib_rtc_status |= LIB_RTC_LOW_POWER | LIB_RTC_OPEN;
	TA0CTL = 0;
#if EN_TA2_PWM    	
	TA2CTL = 0;
#else
	TA1CTL = 0;
#endif
	__set_interrupt_state(state); 	

}

//--------------------------------------------------------------------------------------------------------------------
void rtc_lib_close(void)
{
	istate_t state; 
	state = __get_interrupt_state();
	__disable_interrupt();

	lib_rtc_status = 0;
	TA0CTL = 0;	
#if EN_TA2_PWM    	
	TA2CTL = 0;
#else	
	TA1CTL = 0;
#endif	  
	__set_interrupt_state(state); 

}

//--------------------------------------------------------------------------------------------------------------------
void rtc_lib_open(void)
{ 	
	istate_t state; 
	int32_t half_accumulate_error;

	state = __get_interrupt_state();
	__disable_interrupt();	

	adc_start_flag = 0;
	
  current_rtc_correction = crystal_base_error;
  current_accumulate_error = 0;
  estimated_num_of_aclk_per_second = ACLK_PER_SECOND;
  
  // TA0 used to update software calindar an sample temperature
  TA0CTL = TASSEL_1 + MC_1 + TACLR;         // ACLK, upmode, clear TAR      
  TA0CCTL0 |= CCIE;
	// majority of 1s which be countered by ACLK  
  TA0CCR0 = estimated_num_of_aclk_per_second; 
  //---------------------------------------------
  TA0CCTL1 |= CCIE;
  // Capture for bad delay when set 1s pulse
  TA0CCR1 = estimated_num_of_aclk_per_second - TIME_FOR_BAD_DELAY;  
  //---------------------------------------------
  TA0CCTL2 |= CCIE;
  half_accumulate_error = current_accumulate_error + ((CRYSTAL_BASE_CYCLES_PER_SECOND + current_rtc_correction)>>1);
	old_accumulate_error = half_accumulate_error & 0x1F;
	half_num_of_aclk_per_second = (half_accumulate_error >> 5)-1;
	TA0CCR2 = half_num_of_aclk_per_second - TIME_FOR_BAD_DELAY;

  //TA0CCR2 = (estimated_num_of_aclk_per_second >> 1) - TIME_FOR_BAD_DELAY;   
  //TA0CCTL2 = CAP | CM_1 | CCIS_1;  					// Capture, rising edge, ACLK  

#if EN_TA2_PWM  
  // TA2 used to generate 1s pulse
  TA2CTL = TASSEL_2 | MC_2 | ID_2 | TACLR;  // SMCLK, continue mode, clear TAR 
  TA2CCTL1 = CAP | CM_1 | CCIS_1;  					// Capture, rising edge, ACLK  	
  TA2CCTL0 = OUTMOD0;    
#else
  // TA1 used to generate 1s pulse
  TA1CTL = TASSEL_2 | MC_2 | ID_2 | TACLR;  // SMCLK, continue mode, clear TAR 
  TA1CCTL1 = CAP | CM_1 | CCIS_1;  					// Capture, rising edge, ACLK  	
  TA1CCTL0 = OUTMOD0;    	
#endif
	//pulse_pin_setting();
	
  temperature_coeff_setting();							// Set temperature measurement parameter
	adc10_init(); 
	lib_rtc_status |= LIB_RTC_OPEN;	
	__set_interrupt_state(state); 	
}

#pragma vector=TIMER0_A0_VECTOR
__interrupt void aclk_timer_interrupt(void)
{      
#if defined(SOFTWARE_RTC_SUPPORT)    
  update_rtc();  
	//start temperature conversion
  ADC10CTL0 |= ADC10SC;     
#endif    
}
//--------------------------------------------------------------------------------------

#pragma vector=TIMER0_A1_VECTOR
__interrupt void bad_delay_interrupt(void)
{    
	uint16_t count_aclk1,count_aclk2,count_smclk1,count_smclk2,loop; //,aclk_1s_half;

	for (loop = 0; loop < 100; loop++)
	{
		// Capture smclk number according to aclk
#if EN_TA2_PWM   		
		count_smclk1 = TA2CCR1;
#else
		count_smclk1 = TA1CCR1;
#endif
		// Capture aclk number
		count_aclk1 = TA0R;
		
		// Capture smclk number according to aclk
#if EN_TA2_PWM   			
		count_smclk2 = TA2CCR1;
#else
		count_smclk2 = TA1CCR1;
#endif
		// Capture aclk number	
		count_aclk2 = TA0R; 
		
		// keep the value update synchronization
		if ((count_smclk1 == count_smclk2) && (count_aclk1 == count_aclk2))
			break;
	}
	switch (__even_in_range(TA0IV, 14))
	{
		case	TA0IV_TA0CCR1: 
			//adjust_1s_pulse();		
			//----------------------
#if EN_TA2_PWM    
			TA2CCR0 = count_smclk2 + ((estimated_num_of_aclk_per_second - count_aclk2) << 5) + 272 + current_accumulate_error; 
			
			TA2CCTL0 &= ~OUTMOD2; 
			TA2CCTL0 &= ~CCIFG;
			TA2CCTL0 |= CCIE; 	//switch on TA2CCR0 interrupt, in which we sent failing edge of 1s pulse	
#else
			TA1CCR0 = count_smclk2 + ((estimated_num_of_aclk_per_second - count_aclk2) << 5) + 272 + current_accumulate_error; 
       
			TA1CCTL0 &= ~OUTMOD2; 
			TA1CCTL0 &= ~CCIFG;
			TA1CCTL0 |= CCIE; 	//switch on TA1CCR0 interrupt, in which we sent failing edge of 1s pulse	
#endif
			break; 				 // TA0CCR1_CCIFG
		case	TA0IV_TA0CCR2: 
			//adjust_halfs_pulse();
			//----------------------		
#if EN_TA2_PWM 
			TA2CCR0 = count_smclk2 + ((half_num_of_aclk_per_second - count_aclk2) << 5) + 272 + old_accumulate_error; 

			TA2CCTL0 |= OUTMOD2;
			TA2CCTL0 &= ~CCIFG;
			TA2CCTL0 |= CCIE; 	//switch on TA2CCR0 interrupt, in which we sent failing edge of 1s pulse	
#else
			TA1CCR0 = count_smclk2 + ((half_num_of_aclk_per_second - count_aclk2) << 5) + 272 + old_accumulate_error;  

			TA1CCTL0 |= OUTMOD2;
			TA1CCTL0 &= ~CCIFG;
			TA1CCTL0 |= CCIE; 	//switch on TA1CCR0 interrupt, in which we sent failing edge of 1s pulse	
#endif	

			break; 				 // TA0CCR2_CCIFG
		default: return;
	}
}

//--------------------------------------------------------------------------------------
#if EN_TA2_PWM  
#pragma vector=TIMER2_A0_VECTOR
#else
#pragma vector=TIMER1_A0_VECTOR
#endif
__interrupt void second_pulse_interrupt(void)
{      
  if(lib_rtc_status & LIB_RTC_1SPULSE){
    lib_rtc_status &= ~LIB_RTC_1SPULSE;
  } else {
    adjust_1s_pulse();
    adjust_halfs_pulse();
    lib_rtc_status |= LIB_RTC_1SPULSE;
  }
#if EN_TA2_PWM    
	TA2CCTL0 &= ~CCIE;		//switch off TA2CCR0 interrupt
#else
	TA1CCTL0 &= ~CCIE;		//switch off TA1CCR0 interrupt
#endif

  
}

